function noop() { }
function run(fn) {
    return fn();
}
function blank_object() {
    return Object.create(null);
}
function run_all(fns) {
    fns.forEach(run);
}
function is_function(thing) {
    return typeof thing === 'function';
}
function safe_not_equal(a, b) {
    return a != a ? b == b : a !== b || ((a && typeof a === 'object') || typeof a === 'function');
}
function is_empty(obj) {
    return Object.keys(obj).length === 0;
}

function append(target, node) {
    target.appendChild(node);
}
function insert(target, node, anchor) {
    target.insertBefore(node, anchor || null);
}
function detach(node) {
    node.parentNode.removeChild(node);
}
function destroy_each(iterations, detaching) {
    for (let i = 0; i < iterations.length; i += 1) {
        if (iterations[i])
            iterations[i].d(detaching);
    }
}
function element(name) {
    return document.createElement(name);
}
function text(data) {
    return document.createTextNode(data);
}
function empty() {
    return text('');
}
function attr(node, attribute, value) {
    if (value == null)
        node.removeAttribute(attribute);
    else if (node.getAttribute(attribute) !== value)
        node.setAttribute(attribute, value);
}
function children(element) {
    return Array.from(element.childNodes);
}
function set_data(text, data) {
    data = '' + data;
    if (text.wholeText !== data)
        text.data = data;
}

let current_component;
function set_current_component(component) {
    current_component = component;
}

const dirty_components = [];
const binding_callbacks = [];
const render_callbacks = [];
const flush_callbacks = [];
const resolved_promise = Promise.resolve();
let update_scheduled = false;
function schedule_update() {
    if (!update_scheduled) {
        update_scheduled = true;
        resolved_promise.then(flush);
    }
}
function add_render_callback(fn) {
    render_callbacks.push(fn);
}
let flushing = false;
const seen_callbacks = new Set();
function flush() {
    if (flushing)
        return;
    flushing = true;
    do {
        // first, call beforeUpdate functions
        // and update components
        for (let i = 0; i < dirty_components.length; i += 1) {
            const component = dirty_components[i];
            set_current_component(component);
            update(component.$$);
        }
        set_current_component(null);
        dirty_components.length = 0;
        while (binding_callbacks.length)
            binding_callbacks.pop()();
        // then, once components are updated, call
        // afterUpdate functions. This may cause
        // subsequent updates...
        for (let i = 0; i < render_callbacks.length; i += 1) {
            const callback = render_callbacks[i];
            if (!seen_callbacks.has(callback)) {
                // ...so guard against infinite loops
                seen_callbacks.add(callback);
                callback();
            }
        }
        render_callbacks.length = 0;
    } while (dirty_components.length);
    while (flush_callbacks.length) {
        flush_callbacks.pop()();
    }
    update_scheduled = false;
    flushing = false;
    seen_callbacks.clear();
}
function update($$) {
    if ($$.fragment !== null) {
        $$.update();
        run_all($$.before_update);
        const dirty = $$.dirty;
        $$.dirty = [-1];
        $$.fragment && $$.fragment.p($$.ctx, dirty);
        $$.after_update.forEach(add_render_callback);
    }
}
const outroing = new Set();
function transition_in(block, local) {
    if (block && block.i) {
        outroing.delete(block);
        block.i(local);
    }
}
function mount_component(component, target, anchor) {
    const { fragment, on_mount, on_destroy, after_update } = component.$$;
    fragment && fragment.m(target, anchor);
    // onMount happens before the initial afterUpdate
    add_render_callback(() => {
        const new_on_destroy = on_mount.map(run).filter(is_function);
        if (on_destroy) {
            on_destroy.push(...new_on_destroy);
        }
        else {
            // Edge case - component was destroyed immediately,
            // most likely as a result of a binding initialising
            run_all(new_on_destroy);
        }
        component.$$.on_mount = [];
    });
    after_update.forEach(add_render_callback);
}
function destroy_component(component, detaching) {
    const $$ = component.$$;
    if ($$.fragment !== null) {
        run_all($$.on_destroy);
        $$.fragment && $$.fragment.d(detaching);
        // TODO null out other refs, including component.$$ (but need to
        // preserve final state?)
        $$.on_destroy = $$.fragment = null;
        $$.ctx = [];
    }
}
function make_dirty(component, i) {
    if (component.$$.dirty[0] === -1) {
        dirty_components.push(component);
        schedule_update();
        component.$$.dirty.fill(0);
    }
    component.$$.dirty[(i / 31) | 0] |= (1 << (i % 31));
}
function init(component, options, instance, create_fragment, not_equal, props, dirty = [-1]) {
    const parent_component = current_component;
    set_current_component(component);
    const prop_values = options.props || {};
    const $$ = component.$$ = {
        fragment: null,
        ctx: null,
        // state
        props,
        update: noop,
        not_equal,
        bound: blank_object(),
        // lifecycle
        on_mount: [],
        on_destroy: [],
        before_update: [],
        after_update: [],
        context: new Map(parent_component ? parent_component.$$.context : []),
        // everything else
        callbacks: blank_object(),
        dirty,
        skip_bound: false
    };
    let ready = false;
    $$.ctx = instance
        ? instance(component, prop_values, (i, ret, ...rest) => {
            const value = rest.length ? rest[0] : ret;
            if ($$.ctx && not_equal($$.ctx[i], $$.ctx[i] = value)) {
                if (!$$.skip_bound && $$.bound[i])
                    $$.bound[i](value);
                if (ready)
                    make_dirty(component, i);
            }
            return ret;
        })
        : [];
    $$.update();
    ready = true;
    run_all($$.before_update);
    // `false` as a special case of no DOM component
    $$.fragment = create_fragment ? create_fragment($$.ctx) : false;
    if (options.target) {
        if (options.hydrate) {
            const nodes = children(options.target);
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            $$.fragment && $$.fragment.l(nodes);
            nodes.forEach(detach);
        }
        else {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            $$.fragment && $$.fragment.c();
        }
        if (options.intro)
            transition_in(component.$$.fragment);
        mount_component(component, options.target, options.anchor);
        flush();
    }
    set_current_component(parent_component);
}
class SvelteComponent {
    $destroy() {
        destroy_component(this, 1);
        this.$destroy = noop;
    }
    $on(type, callback) {
        const callbacks = (this.$$.callbacks[type] || (this.$$.callbacks[type] = []));
        callbacks.push(callback);
        return () => {
            const index = callbacks.indexOf(callback);
            if (index !== -1)
                callbacks.splice(index, 1);
        };
    }
    $set($$props) {
        if (this.$$set && !is_empty($$props)) {
            this.$$.skip_bound = true;
            this.$$set($$props);
            this.$$.skip_bound = false;
        }
    }
}

/* src\Message.svelte generated by Svelte v3.29.7 */

function add_css() {
	var style = element("style");
	style.id = "svelte-y8sf20-style";
	style.textContent = "p.valid.svelte-y8sf20{display:none}p.invalid.svelte-y8sf20{color:red}";
	append(document.head, style);
}

function get_each_context(ctx, list, i) {
	const child_ctx = ctx.slice();
	child_ctx[2] = list[i];
	return child_ctx;
}

// (6:0) {#if result}
function create_if_block(ctx) {
	let each_1_anchor;
	let each_value = /*result*/ ctx[0];
	let each_blocks = [];

	for (let i = 0; i < each_value.length; i += 1) {
		each_blocks[i] = create_each_block(get_each_context(ctx, each_value, i));
	}

	return {
		c() {
			for (let i = 0; i < each_blocks.length; i += 1) {
				each_blocks[i].c();
			}

			each_1_anchor = empty();
		},
		m(target, anchor) {
			for (let i = 0; i < each_blocks.length; i += 1) {
				each_blocks[i].m(target, anchor);
			}

			insert(target, each_1_anchor, anchor);
		},
		p(ctx, dirty) {
			if (dirty & /*result, name*/ 3) {
				each_value = /*result*/ ctx[0];
				let i;

				for (i = 0; i < each_value.length; i += 1) {
					const child_ctx = get_each_context(ctx, each_value, i);

					if (each_blocks[i]) {
						each_blocks[i].p(child_ctx, dirty);
					} else {
						each_blocks[i] = create_each_block(child_ctx);
						each_blocks[i].c();
						each_blocks[i].m(each_1_anchor.parentNode, each_1_anchor);
					}
				}

				for (; i < each_blocks.length; i += 1) {
					each_blocks[i].d(1);
				}

				each_blocks.length = each_value.length;
			}
		},
		d(detaching) {
			destroy_each(each_blocks, detaching);
			if (detaching) detach(each_1_anchor);
		}
	};
}

// (8:8) {#if item.field == name}
function create_if_block_1(ctx) {
	let if_block_anchor;

	function select_block_type(ctx, dirty) {
		if (/*item*/ ctx[2].message === "") return create_if_block_2;
		return create_else_block;
	}

	let current_block_type = select_block_type(ctx);
	let if_block = current_block_type(ctx);

	return {
		c() {
			if_block.c();
			if_block_anchor = empty();
		},
		m(target, anchor) {
			if_block.m(target, anchor);
			insert(target, if_block_anchor, anchor);
		},
		p(ctx, dirty) {
			if (current_block_type === (current_block_type = select_block_type(ctx)) && if_block) {
				if_block.p(ctx, dirty);
			} else {
				if_block.d(1);
				if_block = current_block_type(ctx);

				if (if_block) {
					if_block.c();
					if_block.m(if_block_anchor.parentNode, if_block_anchor);
				}
			}
		},
		d(detaching) {
			if_block.d(detaching);
			if (detaching) detach(if_block_anchor);
		}
	};
}

// (11:12) {:else}
function create_else_block(ctx) {
	let p;
	let t_value = /*item*/ ctx[2].message + "";
	let t;

	return {
		c() {
			p = element("p");
			t = text(t_value);
			attr(p, "class", "invalid svelte-y8sf20");
		},
		m(target, anchor) {
			insert(target, p, anchor);
			append(p, t);
		},
		p(ctx, dirty) {
			if (dirty & /*result*/ 1 && t_value !== (t_value = /*item*/ ctx[2].message + "")) set_data(t, t_value);
		},
		d(detaching) {
			if (detaching) detach(p);
		}
	};
}

// (9:12) {#if item.message === ''}
function create_if_block_2(ctx) {
	let p;

	return {
		c() {
			p = element("p");
			attr(p, "class", "valid svelte-y8sf20");
		},
		m(target, anchor) {
			insert(target, p, anchor);
		},
		p: noop,
		d(detaching) {
			if (detaching) detach(p);
		}
	};
}

// (7:4) {#each result as item}
function create_each_block(ctx) {
	let if_block_anchor;
	let if_block = /*item*/ ctx[2].field == /*name*/ ctx[1] && create_if_block_1(ctx);

	return {
		c() {
			if (if_block) if_block.c();
			if_block_anchor = empty();
		},
		m(target, anchor) {
			if (if_block) if_block.m(target, anchor);
			insert(target, if_block_anchor, anchor);
		},
		p(ctx, dirty) {
			if (/*item*/ ctx[2].field == /*name*/ ctx[1]) {
				if (if_block) {
					if_block.p(ctx, dirty);
				} else {
					if_block = create_if_block_1(ctx);
					if_block.c();
					if_block.m(if_block_anchor.parentNode, if_block_anchor);
				}
			} else if (if_block) {
				if_block.d(1);
				if_block = null;
			}
		},
		d(detaching) {
			if (if_block) if_block.d(detaching);
			if (detaching) detach(if_block_anchor);
		}
	};
}

function create_fragment(ctx) {
	let if_block_anchor;
	let if_block = /*result*/ ctx[0] && create_if_block(ctx);

	return {
		c() {
			if (if_block) if_block.c();
			if_block_anchor = empty();
		},
		m(target, anchor) {
			if (if_block) if_block.m(target, anchor);
			insert(target, if_block_anchor, anchor);
		},
		p(ctx, [dirty]) {
			if (/*result*/ ctx[0]) {
				if (if_block) {
					if_block.p(ctx, dirty);
				} else {
					if_block = create_if_block(ctx);
					if_block.c();
					if_block.m(if_block_anchor.parentNode, if_block_anchor);
				}
			} else if (if_block) {
				if_block.d(1);
				if_block = null;
			}
		},
		i: noop,
		o: noop,
		d(detaching) {
			if (if_block) if_block.d(detaching);
			if (detaching) detach(if_block_anchor);
		}
	};
}

function instance($$self, $$props, $$invalidate) {
	let { result } = $$props;
	let { name } = $$props;

	$$self.$$set = $$props => {
		if ("result" in $$props) $$invalidate(0, result = $$props.result);
		if ("name" in $$props) $$invalidate(1, name = $$props.name);
	};

	return [result, name];
}

class Message extends SvelteComponent {
	constructor(options) {
		super();
		if (!document.getElementById("svelte-y8sf20-style")) add_css();
		init(this, options, instance, create_fragment, safe_not_equal, { result: 0, name: 1 });
	}
}

/* src\AllMessages.svelte generated by Svelte v3.29.7 */

function add_css$1() {
	var style = element("style");
	style.id = "svelte-m6tmye-style";
	style.textContent = "p.invalid.svelte-m6tmye{margin:3px 0px;font-size:14px;color:red}";
	append(document.head, style);
}

function get_each_context$1(ctx, list, i) {
	const child_ctx = ctx.slice();
	child_ctx[1] = list[i];
	return child_ctx;
}

// (5:0) {#if result}
function create_if_block$1(ctx) {
	let each_1_anchor;
	let each_value = /*result*/ ctx[0];
	let each_blocks = [];

	for (let i = 0; i < each_value.length; i += 1) {
		each_blocks[i] = create_each_block$1(get_each_context$1(ctx, each_value, i));
	}

	return {
		c() {
			for (let i = 0; i < each_blocks.length; i += 1) {
				each_blocks[i].c();
			}

			each_1_anchor = empty();
		},
		m(target, anchor) {
			for (let i = 0; i < each_blocks.length; i += 1) {
				each_blocks[i].m(target, anchor);
			}

			insert(target, each_1_anchor, anchor);
		},
		p(ctx, dirty) {
			if (dirty & /*result*/ 1) {
				each_value = /*result*/ ctx[0];
				let i;

				for (i = 0; i < each_value.length; i += 1) {
					const child_ctx = get_each_context$1(ctx, each_value, i);

					if (each_blocks[i]) {
						each_blocks[i].p(child_ctx, dirty);
					} else {
						each_blocks[i] = create_each_block$1(child_ctx);
						each_blocks[i].c();
						each_blocks[i].m(each_1_anchor.parentNode, each_1_anchor);
					}
				}

				for (; i < each_blocks.length; i += 1) {
					each_blocks[i].d(1);
				}

				each_blocks.length = each_value.length;
			}
		},
		d(detaching) {
			destroy_each(each_blocks, detaching);
			if (detaching) detach(each_1_anchor);
		}
	};
}

// (6:4) {#each result as item}
function create_each_block$1(ctx) {
	let p;
	let t_value = /*item*/ ctx[1].message + "";
	let t;

	return {
		c() {
			p = element("p");
			t = text(t_value);
			attr(p, "class", "invalid svelte-m6tmye");
		},
		m(target, anchor) {
			insert(target, p, anchor);
			append(p, t);
		},
		p(ctx, dirty) {
			if (dirty & /*result*/ 1 && t_value !== (t_value = /*item*/ ctx[1].message + "")) set_data(t, t_value);
		},
		d(detaching) {
			if (detaching) detach(p);
		}
	};
}

function create_fragment$1(ctx) {
	let if_block_anchor;
	let if_block = /*result*/ ctx[0] && create_if_block$1(ctx);

	return {
		c() {
			if (if_block) if_block.c();
			if_block_anchor = empty();
		},
		m(target, anchor) {
			if (if_block) if_block.m(target, anchor);
			insert(target, if_block_anchor, anchor);
		},
		p(ctx, [dirty]) {
			if (/*result*/ ctx[0]) {
				if (if_block) {
					if_block.p(ctx, dirty);
				} else {
					if_block = create_if_block$1(ctx);
					if_block.c();
					if_block.m(if_block_anchor.parentNode, if_block_anchor);
				}
			} else if (if_block) {
				if_block.d(1);
				if_block = null;
			}
		},
		i: noop,
		o: noop,
		d(detaching) {
			if (if_block) if_block.d(detaching);
			if (detaching) detach(if_block_anchor);
		}
	};
}

function instance$1($$self, $$props, $$invalidate) {
	let { result } = $$props;

	$$self.$$set = $$props => {
		if ("result" in $$props) $$invalidate(0, result = $$props.result);
	};

	return [result];
}

class AllMessages extends SvelteComponent {
	constructor(options) {
		super();
		if (!document.getElementById("svelte-m6tmye-style")) add_css$1();
		init(this, options, instance$1, create_fragment$1, safe_not_equal, { result: 0 });
	}
}

/*
Author: Kamyar Lajani
Email: kamolajani1@gmail.com
*/

function isEmailValid(email) {
    let mailformat = /[a-z0-9!#$%&'*+/=?^_`{|}~-]+(?:\.[a-z0-9!#$%&'*+/=?^_`{|}~-]+)*@(?:[a-z0-9](?:[a-z0-9-]*[a-z0-9])?\.)+[a-z0-9](?:[a-z0-9-]*[a-z0-9])?/;
    if(email.match(mailformat)){
        return true;
    }
    return false;
}
function isValidDate(dateString, dateFormat, startDate, endDate) {
    let datePattern;
    let year, month, day;
    if(dateFormat === 'yyyy-mm-dd') {
        datePattern = /^([12]\d{3}-(0[1-9]|1[0-2])-(0[1-9]|[12]\d|3[01]))/;
         // Remove any non digit characters
        const cleanDateString = dateString.replace(/\D/g, ''); 
        // Parse integer values from date string
        year = parseInt(cleanDateString.substr(0, 4));
        month = parseInt(cleanDateString.substr(4, 2));
        day = parseInt(cleanDateString.substr(6, 2));
    }
    else if(dateFormat === 'yyyy/mm/dd') {
        datePattern = /^([12]\d{3}\/(0[1-9]|1[0-2])\/(0[1-9]|[12]\d|3[01]))/;
        const cleanDateString = dateString.replace(/\D/g, ''); 
        year = parseInt(cleanDateString.substr(0, 4));
        month = parseInt(cleanDateString.substr(4, 2));
        day = parseInt(cleanDateString.substr(6, 2));
    }
    else if(dateFormat === 'dd-mm-yyyy') {
        datePattern = /^(0[1-9]|[12][0-9]|3[01])-(0[1-9]|1[012])-(19|20)\d\d$/;
        const cleanDateString = dateString.replace(/\D/g, ''); 
        year = parseInt(cleanDateString.substr(4, 4));
        month = parseInt(cleanDateString.substr(2, 2));
        day = parseInt(cleanDateString.substr(0, 2));
    }
    else if(dateFormat === 'dd/mm/yyyy') {
        datePattern = /^(0[1-9]|[12][0-9]|3[01])\/(0[1-9]|1[012])\/(19|20)\d\d$/;
        const cleanDateString = dateString.replace(/\D/g, ''); 
        year = parseInt(cleanDateString.substr(4, 4));
        month = parseInt(cleanDateString.substr(2, 2));
        day = parseInt(cleanDateString.substr(0, 2));
    }
    else if(dateFormat === 'mm-dd-yyyy') {
        datePattern = /^(0[1-9]|1[012])-(0[1-9]|[12][0-9]|3[01])-(19|20)\d\d$/;
        const cleanDateString = dateString.replace(/\D/g, ''); 
        year = parseInt(cleanDateString.substr(4, 4));
        month = parseInt(cleanDateString.substr(0, 2));
        day = parseInt(cleanDateString.substr(2, 2));
    }
    else if(dateFormat === 'mm/dd/yyyy') {
        datePattern = /^(0[1-9]|1[012])\/(0[1-9]|[12][0-9]|3[01])\/(19|20)\d\d$/;
        const cleanDateString = dateString.replace(/\D/g, ''); 
        year = parseInt(cleanDateString.substr(4, 4));
        month = parseInt(cleanDateString.substr(0, 2));
        day = parseInt(cleanDateString.substr(2, 2));
    }
    else if(dateFormat === 'mm/dd/yyyy') {
        datePattern = /^(0[1-9]|1[012])\/(0[1-9]|[12][0-9]|3[01])\/(19|20)\d\d$/;
        const cleanDateString = dateString.replace(/\D/g, ''); 
        year = parseInt(cleanDateString.substr(4, 4));
        month = parseInt(cleanDateString.substr(0, 2));
        day = parseInt(cleanDateString.substr(2, 2));
    }
    // date ranges
    if(startDate !== undefined){
        if(dateFormat === 'mm/dd/yyyy' || dateFormat === 'mm-dd-yyyy') {
            const cleanDateString = startDate.replace(/\D/g, ''); 
            startYear = parseInt(cleanDateString.substr(4, 4));
            startMonth = parseInt(cleanDateString.substr(0, 2));
            startDay = parseInt(cleanDateString.substr(2, 2));
        }
        else if(dateFormat === 'dd/mm/yyyy' || dateFormat === 'dd-mm-yyyy') {
            const cleanDateString = startDate.replace(/\D/g, ''); 
            startYear = parseInt(cleanDateString.substr(4, 4));
            startMonth = parseInt(cleanDateString.substr(2, 2));
            startDay = parseInt(cleanDateString.substr(0, 2));
        }
        else if(dateFormat === 'yyyy-mm-dd' || dateFormat === 'yyyy/mm/dd') {
            const cleanDateString = startDate.replace(/\D/g, ''); 
            startYear = parseInt(cleanDateString.substr(0, 4));
            startMonth = parseInt(cleanDateString.substr(4, 2));
            startDay = parseInt(cleanDateString.substr(6, 2));
        }
        if(startYear > year) {
            return false;
        }
        else {            if(startMonth > month) {
                return false;
            }
            else {
                if(startDay > day) {
                    return false;
                }
            }
        }
    }
    if(endDate !== undefined){
        if(dateFormat === 'mm/dd/yyyy' || dateFormat === 'mm-dd-yyyy') {
            const cleanDateString = endDate.replace(/\D/g, ''); 
            endYear = parseInt(cleanDateString.substr(4, 4));
            endMonth = parseInt(cleanDateString.substr(0, 2));
            endDay = parseInt(cleanDateString.substr(2, 2));
        }
        else if(dateFormat === 'dd/mm/yyyy' || dateFormat === 'dd-mm-yyyy') {
            const cleanDateString = endDate.replace(/\D/g, ''); 
            endYear = parseInt(cleanDateString.substr(4, 4));
            endMonth = parseInt(cleanDateString.substr(2, 2));
            endDay = parseInt(cleanDateString.substr(0, 2));
        }
        else if(dateFormat === 'yyyy-mm-dd' || dateFormat === 'yyyy/mm/dd') {
            const cleanDateString = endDate.replace(/\D/g, ''); 
            endYear = parseInt(cleanDateString.substr(0, 4));
            endMonth = parseInt(cleanDateString.substr(4, 2));
            endDay = parseInt(cleanDateString.substr(6, 2));
        }
        if(endYear < year) {
            return false;
        }
        else {
            if(endMonth < month) {
                return false;
            }
            else {
                if(endDay < day) {
                    return false;
                }
            }
        }
    }
    let matchArray = dateString.match(datePattern);
    if (matchArray == null) {
        return false;
    }
    // Define number of days per month
    let daysInMonth = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];
    // Adjust for leap years
    if (year % 400 == 0 || (year % 100 != 0 && year % 4 == 0)) {
        daysInMonth[1] = 29;
    }
    // check month and day range
    if (month < 1 || month > 12 || day < 1 || day > daysInMonth[month - 1]) {
        return false;
    }
    return true;
}
function isValidTime(time){
    const timeFormat = /^([0-1][0-9]|2[0-3]):([0-5][0-9])$/;
    if(time.match(timeFormat)){
        return true;
    }
    return false;
}
let validoz = (fields)=>{
    let errors = [];
    // is fields multiple?
    if(Array.isArray(fields)){
        for(let field of fields) {
            errors.push(validate(field));
        }
        return errors;
    }
    else if(typeof fields === 'object' && fields !== null){
        return validate(fields);
    }
};
const validate = (field)=>{
    let error = [];
    if(!field.name || field.name === ''){
        throw new Error(`Property name not passed`);
    }
    if(!field.type || field.type === ''){
        throw new Error(`Property ${field.name} type not passed`);
    }
    else if(field.value === undefined){
        throw new Error(`Property value not passed`);
    }
    else if(field.value === '' && (field.required === undefined || field.required === true)){
        error.push(`${field.name} is required`);
    }
    // minLength & maxLength
    if(field.minLength && !field.maxLength){
        if(!Number.isInteger(field.minLength)){
            throw new Error(`minLength is not an integer`);
        }
        else if(field.minLength < 1){
            throw new Error(`minLength value must be at least 1`);
        }
        else if(field.value.length < field.minLength){
            error.push(`${field.name} length must be at least ${field.minLength} characters`);
        }
    }
    else if(!field.minLength && field.maxLength){
        if(!Number.isInteger(field.maxLength)){
            throw new Error(`maxLength is not an integer`);
        }
        else if(field.value.length > field.maxLength){
            error.push(`${field.name} length must be less than ${field.maxLength} characters`);
        }
    }
    else if(field.minLength && field.maxLength) {
        if(!Number.isInteger(field.maxLength) || !Number.isInteger(field.minLength)){
            throw new Error(`minLength and maxLength must be an integer`);
        }
        else if(field.value.length < field.minLength || field.value.length > field.maxLength){
            error.push(`${field.name} length must be between ${field.minLength} and ${field.maxLength}`);
        }
    }
    if(field.type === 'number'){
        const value = parseInt(field.value);
        if(!Number.isInteger(value) && field.value != ''){
            error.push('value must be a number');
        }
        else {
            // min & max
            if(field.min && !field.max){
                if(!Number.isInteger(field.min)){
                    throw new Error(`min is not an integer`);
                }
                else if(value < field.min){
                    error.push(`${field.name} must be at least ${field.min}`);
                }
            }
            if(!field.min && field.max){
                if(!Number.isInteger(field.max)){
                    throw new Error(`max is not an integer`);
                }
                else if(value > field.max){
                    error.push(`${field.name} must be less than ${field.max}`);
                }
            }
            else if(field.min && field.max){
                if(!Number.isInteger(field.max) || !Number.isInteger(field.min)){
                    throw new Error(`min and max must be an integer`);
                }
                else if(value < field.min || value > field.max){
                    error.push(`${field.name} must be between ${field.min} and ${field.max}`);
                }
            }
            // minDigits & maxDigits
            if(field.minDigits && !field.maxDigits){
                if(!Number.isInteger(field.minDigits)){
                    throw new Error(`minDigits is not an integer`);
                }
                else if(value.toString().length < field.minDigits){
                    error.push(`${field.name} must be at least ${field.minDigits} digits`);
                }
            }
            else if(!field.minDigits && field.maxDigits){
                if(!Number.isInteger(field.maxDigits)){
                    throw new Error(`maxDigits is not an integer`);
                }
                else if(value.toString().length > field.maxDigits){
                    error.push(`${field.name} must be less than ${field.maxDigits} digits`);
                }
            }
            else if(field.minDigits && field.maxDigits){
                if(!Number.isInteger(field.maxDigits) || !Number.isInteger(field.minDigits)){
                    throw new Error(`minDigits and maxDigits must be an integer`);
                }
                else if(value.toString().length < field.minDigits || value.toString().length > field.maxDigits){
                    error.push(`${field.name} must be between ${field.minDigits} and ${field.maxDigits} digits`);
                }
            }
        }
    }
    if(field.type === 'email'){
        if(!isEmailValid(field.value)){
            error.push(`Email address is invalid`);
        }
    }
    if(field.type === 'date'){
        if(!isValidDate(field.value, field.dateFormat, field.startDate, field.endDate)){
            error.push(`${field.name} is invalid`);
        }
    }
    if(field.type === 'time'){
        if(!isValidTime(field.value)){
            error.push(`${field.name} is invalid`);
        }
    }
    if(field.type === 'username'){
        const usernameFormat = /^[a-zA-Z0-9_]{5,}[a-zA-Z]+[0-9]*$/;
        if(!field.value.match(usernameFormat)){
            error.push(`${field.name} is invalid`);
        }
    }
    if(field.type === 'fullname'){
        const fullnameFormat = /^([\w]{3,})+\s+([\w\s]{3,})+$/i;
        if(!field.value.match(fullnameFormat)){
            error.push(`${field.name} is invalid`);
        }
    }
    if(field.type === 'word'){
        const alphabetFormat = /^[A-Za-z]+$/;
        if(!field.value.match(alphabetFormat)){
            error.push(`${field.name} is invalid`);
        }
    }
    if(field.type === 'password'){
        const passwordFormat = /^(?=.*\d)(?=.*[a-z])(?=.*[A-Z])[0-9a-zA-Z]{6,}$/;
        if(!field.value.match(passwordFormat)){
            error.push(`${field.name} must contain at least one numberic, one upper case and one lower case characters and the length at least 6 characters`);
        }
    }
    if(field.equal !== undefined){
        if(field.value !== field.equal){
            error.push(`${field.name} is wrong`);
        }
    }
    if(field.notEqual !== undefined){
        if(field.value === field.notEqual){
            error.push(`${field.name} is wrong`);
        }
    }
    if(error[0] === undefined){
        return {field: field.name, message: ''};
    }
    return {field: field.name, message: error[0]};
};

let isValid = (fields) => {
    // is fields multiple?
    let isValid = true;
    if(Array.isArray(fields)){
        for(let field of fields) {
            if(field.message !== ''){
                isValid = false;
            }
        }
    }
    else if(typeof fields === 'object' && fields !== null){
        if(fields.message !== ''){
            isValid = false;
        }
    }
    return isValid;
};
let isValidByName = (fields, name) => {
    // is fields multiple?
    let isValid = true;
    if(Array.isArray(fields)){
        for(let field of fields) {
            if(field.message !== '' && field.field === name){
                isValid = false;
            }
        }
    }
    else if(typeof fields === 'object' && fields !== null){
        if(fields.message !== '' && fields.field === name){
            isValid = false;
        }
    }
    return isValid;
};

export { AllMessages, Message, isValid, isValidByName, validoz };
